@@ -3,42 +3,85 @@ svmlight<-function (x, ...)
 
 
 svmlight.default <- function(x, grouping, temp.dir=NULL, pathsvm=NULL, del=TRUE, 
-    svm.options=NULL, prior=NULL, out=FALSE, ...)
+    type="C", class.type = "oaa" ,svm.options=NULL,  prior=NULL, out=FALSE, ...)
 {
-    y <- grouping
+y <- grouping
+ pick<-NULL     
+ if (type!="R")
+   {
+    ### Construct Dummymatrix for 1-a-a classification: 
+    ### ncol= no. of classes, (i,j)=+1 iff obj i comes from class j, -1 else 
     if(is.null(prior)) prior <- table(y) / length(y)
-
-### Construct Dummymatrix for 1-a-a classification: 
-### ncol= no. of classes, (i,j)=+1 iff obj i comes from class j, -1 else 
     ys <- as.factor(y)
     tys <- table(ys)
-    ymat <- matrix(-1, nrow = nrow(x), ncol = length(tys))
     lev <- levels(ys)
-    ymat[cbind(seq(along = ys), sapply(ys, function(x) which(x == lev)))] <- 1
-    counts <- as.vector(tys)
+    if (class.type !="oao")
+      {
+       class.type<-"oaa"
+       ymat <- matrix(-1, nrow = nrow(x), ncol = length(tys))
+       ymat[cbind(seq(along = ys), sapply(ys, function(x) which(x == lev)))] <- 1
+      }
+     else
+       { ## Classification: one against one
+        nclass <- length(table(levels(y)))
+        m <- (nclass - 1)
+        minus <- nclass + 1 - sequence(m:1)
+        plus <- rep(1:m, m:1)
+        pick <- rbind(plus, minus)
+        xsplit <- split(data.frame(x), ys)
+        ymat <- list()
+        xlist <- list()
+        for(k in 1:ncol(pick)){
+            ymat[[k]] <- c(rep(1, nrow(xsplit[[ pick[1, k] ]])), rep(-1, nrow(xsplit[[ pick[2, k] ]])))
+            xlist[[k]] <- rbind(xsplit[[ pick[1, k] ]], xsplit[[ pick[2, k] ]])
+            }
+        } 
+    counts <- as.vector(tys)     
+  }
+ else
+   {
+   ### Regression: So transform vector to 1-col matrix and set options, J ...
+     ymat <- matrix(y,ncol=1)
+     lev <- NULL
+     counts <- 1
+     svm.options <- paste("-z r ",svm.options)
+     J <- 1 
+     }
     svm.model <- list()
+
 ### call svm_learn
-    cmd <- if(is.null(pathsvm)) "svm_learn" else file.path(pathsvm, "svm_learn")
-    J <- 1:ncol(ymat)
+    cmd <- if (is.null(pathsvm)) 
+        "svm_learn"
+    else file.path(pathsvm, "svm_learn")
+    
+    if(is.matrix(ymat)) J <- 1:ncol(ymat)
+    if(is.list(ymat)) J <- 1:length(ymat)
+    
 ### "paste" file names for training data and model
     train.filename <- paste(temp.dir, "_train_", J, ".dat", sep = "")
     model.filename <- paste(temp.dir, "_model_", J, ".txt", sep = "")
     PWin <- .Platform$OS.type == "windows"
-### for all 1-a-a test call svm_learn
-    for (j in J){
-    ### construct matrix to learn svm in a format, so that svmlight can read it
-        train <- svmlight.file(cbind(ymat[,j], x), train = TRUE)    
-    ### save to disk
-        write.table(train, file = train.filename[j], row.names = FALSE, 
-        col.names = FALSE, quote = FALSE)
-        if (PWin) 
-            system(paste(cmd, svm.options, train.filename[j], model.filename[j]), 
-                show.output.on.console = out)
-        else 
-            system(paste(cmd,svm.options, train.filename[j], model.filename[j]))
-    ### store learned model
-        svm.model[[j]] <- readLines(model.filename[j])
-    }
+ for (j in J){
+      ### construct matrix to learn svm in a format, so that svmlight can read it
+      if (class.type !="oao")
+         {
+          train <- svmlight.file(cbind(ymat[,j], x), train = TRUE)    
+          }
+        else
+          {
+           train <- svmlight.file(cbind(ymat[[j]], xlist[[j]]), train = TRUE)
+           }  
+      ### save to disk
+          write.table(train, file = train.filename[j], row.names = FALSE, 
+          col.names = FALSE, quote = FALSE)
+          if (PWin) 
+              system(paste(cmd, svm.options, train.filename[j], model.filename[j]), 
+                  show.output.on.console = out)
+          else 
+              system(paste(cmd,svm.options, train.filename[j], model.filename[j]))
+      ### store learned model
+          svm.model[[j]] <- readLines(model.filename[j])
+      }
     if (del) 
        file.remove(c(train.filename, model.filename))
 
@@ -46,11 +89,11 @@ svmlight.default <- function(x, grouping, temp.dir=NULL, pathsvm=NULL, del=TRUE,
     cl[[1]] <- as.name("svmlight")
     structure(list(prior = prior, counts = counts, lev = lev, 
         temp.dir = temp.dir, pathsvm = pathsvm, del = del, 
+        type=type, class.type=class.type, pick=pick, J=J,
         svm.model = svm.model, svm.options = svm.options, call = cl), class = "svmlight")
 }
 
 
-
 #### svmlight interface for different calls. Copied and adapted from lda.xxx
 svmlight.formula <- function(formula, data = NULL, ..., subset, na.action = na.fail) 
 {
@@ -117,8 +160,21 @@ svmlight.data.frame <- function (x, ...)
 
 
 ### predict method for svmlight
-predict.svmlight <- function(object, newdata, scal = TRUE, ...)
+predict.svmlight <- function(object, newdata, scal=TRUE,...)
 {
+### utility function for oao classification
+sf<-function(x,pick)
+{
+erg<-numeric(max(pick))
+names(erg)<-1:max(pick)
+dummy<-table(diag(pick[(x<0)+1,]))
+erg[names(dummy)]<-dummy
+return(erg)
+}
+
+
+
+
 ### copied and adapted form predict.lda
     if (!inherits(object, "svmlight")) 
         stop("object not of class svmlight")
@@ -151,14 +207,17 @@ predict.svmlight <- function(object, newdata, scal = TRUE, ...)
 #######################################
 
 ### save test data on disk
-    x <- svmlight.file(x, train = FALSE)
+    x <- svmlight.file(cbind(rep(0,nrow(x)),x), train = TRUE)
+    #x <- svmlight.file(x, train = FALSE)
     test.filename <- paste(object$temp.dir, "_test_.dat", sep = "")
     write.table(x, file = test.filename, row.names = FALSE, 
         col.names = FALSE, quote = FALSE)
     werte <- NULL
-    cmd <- if(is.null(object$pathsvm)) "svm_classify" 
-           else file.path(object$pathsvm, "svm_classify")    
-    J <- seq(along = object$counts)
+    
+    cmd <- if (is.null(object$pathsvm)) 
+        "svm_classify"
+    else file.path(object$pathsvm, "svm_classify")
+    J <- object$J
     model.filename <- paste(object$temp.dir, "_model_", J, ".txt", sep = "")
     pred.filename <- paste(object$temp.dir, "_pred_", J, ".txt", sep = "")
 ### for all learned models predict call svm_classify
@@ -171,32 +230,30 @@ predict.svmlight <- function(object, newdata, scal = TRUE, ...)
     }
     if (object$del) 
         file.remove(c(test.filename, pred.filename, model.filename))
-### choose class with highest decision value f(x)
+    if (length(J)>1)
+    {
+    ### Classification: choose class with highest decision value f(x)
+    if  (object$class.type=="oao")   
+        {
+        werte2<-apply(werte,1,sf,pick=object$pick) 
+        werte<-t(werte2)
+        }
+    
+    
+    
     classes <- factor(max.col(werte), levels = seq(along = object$lev), 
         labels = object$lev)
-    colnames(werte) <- object$lev
-    if (scal) werte <- e.scal(werte)$sv
+    colnames(werte)<-object$lev 
+    
+    if (scal) werte<-e.scal(werte)$sv
     return(list(class = classes, posterior = werte))
+  }
+   else {return(as.vector(werte))} ### Regression
 }
 
+
 svmlight.file <- function(x, train = FALSE,...)
 {
-### write data to the following svmlight format. See: http://svmlight.joachims.org/
-# <line> .=. <target> <feature>:<value> <feature>:<value> ... <feature>:<value>
-# <target> .=. +1 | -1 | 0 | <float> 
-# <feature> .=. <integer> | "qid"  ### # qid not supported
-# <value> .=. <float>
-# The target value and each of the feature/value pairs are separated by a space character. 
-# Feature/value pairs MUST be ordered by increasing feature number. Features with value zero can be skipped.
-# In classification mode, the target value denotes the class of the example. +1 as the target value marks a positive example, 
-# -1 a negative example respectively. So, for example, the line
-# -1 1:0.43 3:0.12 9284:0.2
-# specifies a negative example for which feature number 1 has the value 0.43, feature number 3 has the value 0.12, 
-# feature number 9284 has the value 0.2, and all the other features have value 0. 
-# A class label of 0 indicates that this example should be classified using transduction.
-# The predictions for the examples classified by transduction are written to the file specified through the -l option. 
-# The order of the predictions is the same as in the training data. 
-# In regression mode, the <target> contains the real-valued target value.
     if(is.vector(x)) x <- t(x)
     erg <- x
     sn <- 1:nrow(x) 
